from unittest import IsolatedAsyncioTestCase, TestCase
from unittest.mock import sentinel

from fred_api.diagnostics.service_diagnostics_grpc_pb2 import AboutReply, StatusReply
from frgal.utils import AsyncTestClientMixin
from google.protobuf.empty_pb2 import Empty

from diagonal.client import (
    About,
    DiagnosticsClient,
    DiagnosticsDecoder,
    Service,
    Status,
)


class TestDiagnosticsClient(AsyncTestClientMixin, DiagnosticsClient):
    """Test DiagnosticsClient."""


class DiagnosticsDecoderTest(TestCase):
    def setUp(self):
        self.decoder = DiagnosticsDecoder()

    def test_decode_service(self):
        service = StatusReply.Data.Service()
        service.status = 2
        service.note.value = "note"
        service.extras["x"].value = "X"
        service.extras["y"].value = "Y"
        self.assertEqual(
            self.decoder.decode(service),
            Service(status=Status.WARNING, note="note", extras={"x": "X", "y": "Y"}),
        )


class DiagnosticsClientTest(IsolatedAsyncioTestCase):
    """Test DiagnosticsClient's methods."""

    def setUp(self):
        self.client = TestDiagnosticsClient(netloc=sentinel.netloc)

    async def test_about(self):
        about = AboutReply()
        about.data.server_version.value = "1.0.0"
        about.data.api_versions["Diagnostics"].value = "0.1.0"
        about.data.api_versions["AnotherApi"].value = "0.0.1"
        self.client.mock.return_value = about

        self.assertEqual(
            await self.client.about(),
            About(
                server_version="1.0.0",
                api_versions={"Diagnostics": "0.1.0", "AnotherApi": "0.0.1"},
            ),
        )

        self.client.mock.assert_called_once_with(
            Empty(),
            method="/Fred.Diagnostics.Api.Diagnostics/about",
            timeout=None,
        )

    async def test_status(self):
        status = StatusReply()
        status.data.services["Service"].status = 3
        status.data.services["Service"].note.value = "note"
        status.data.services["Service"].extras["extra"].value = "value"
        status.data.services["Diagnostics"].status = 1
        self.client.mock.return_value = status

        self.assertEqual(
            await self.client.status(),
            {
                "Service": Service(
                    status=Status.ERROR, note="note", extras={"extra": "value"}
                ),
                "Diagnostics": Service(status=Status.OK),
            },
        )

        self.client.mock.assert_called_once_with(
            Empty(),
            method="/Fred.Diagnostics.Api.Diagnostics/status",
            timeout=None,
        )
